@@ -48,23 +48,27 @@ use common::sense;
 
 use Carp ();
 use JSON::XS ();
+use AnyEvent ();
 use AnyEvent::HTTP ();
 
-our $VERSION = 0;
+our $VERSION = 0.1;
 
 our $WEB_ELEMENT_IDENTIFIER = "element-6066-11e4-a52e-4f735466cecf";
 
 my $json = JSON::XS->new
-   ->utf8
-   ->boolean_values (0, 1);
+   ->utf8;
+
+$json->boolean_values (0, 1)
+   if $json->can ("boolean_values");
 
 sub req_ {
-   my ($wdf, $method, $ep, $body, $cb) = @_;
+   my ($self, $method, $ep, $body, $cb) = @_;
 
-   AnyEvent::HTTP::http_request $method => "$wdf->{_ep}$ep",
+   AnyEvent::HTTP::http_request $method => "$self->{_ep}$ep",
       body => $body,
+      timeout => $self->{timeout},
       headers => { "content-type" => "application/json; charset=utf-8", "cache-control" => "no-cache" },
-      ($wdf->{proxy} eq "default" ? () : (proxy => $wdf->{proxy})),
+      ($self->{proxy} eq "default" ? () : (proxy => $self->{proxy})),
       sub {
          my ($res, $hdr) = @_;
 
@@ -77,21 +81,21 @@ sub req_ {
 }
 
 sub get_ {
-   my ($wdf, $ep, $cb) = @_;
+   my ($self, $ep, $cb) = @_;
 
-   $wdf->req_ (GET => $ep, undef, $cb)
+   $self->req_ (GET => $ep, undef, $cb)
 }
 
 sub post_ {
-   my ($wdf, $ep, $data, $cb) = @_;
+   my ($self, $ep, $data, $cb) = @_;
 
-   $wdf->req_ (POST => $ep, $json->encode ($data || {}), $cb)
+   $self->req_ (POST => $ep, $json->encode ($data || {}), $cb)
 }
 
 sub delete_ {
-   my ($wdf, $ep, $cb) = @_;
+   my ($self, $ep, $cb) = @_;
 
-   $wdf->req_ (DELETE => $ep, "", $cb)
+   $self->req_ (DELETE => $ep, "", $cb)
 }
 
 sub AUTOLOAD {
@@ -138,7 +142,7 @@ sub AUTOLOAD {
 
 =item new AnyEvent::WebDriver key => value...
 
-Create a new WebDriver object. Example for a remote webdriver connection
+Create a new WebDriver object. Example for a remote WebDriver connection
 (the only type supported at the moment):
 
    my $wd = new AnyEvent::WebDriver host => "localhost", port => 4444;
@@ -164,6 +168,13 @@ If true (the default), then automatically execute C<delete_session> when
 the WebDriver object is destroyed with an active session. IF set to a
 false value, then the session will continue to exist.
 
+=item timeout => $seconds
+
+The HTTP timeout, in (fractional) seconds (default: C<300>, but this will
+likely drastically reduce). This timeout is reset on any activity, so it
+is not an overall request timeout. Also, individual requests might extend
+this timeout if they are known to take longer.
+
 =back
 
 =cut
@@ -175,15 +186,16 @@ sub new {
       endpoint   => "http://localhost:4444",
       proxy      => undef,
       autodelete => 1,
+      timeout    => 300,
       %kv,
    }, $class
 }
 
 sub DESTROY {
-   my ($wdf) = @_;
+   my ($self) = @_;
 
-   $wdf->delete_session
-      if exists $wdf->{sid};
+   $self->delete_session
+      if exists $self->{sid};
 }
 
 =back
@@ -196,11 +208,11 @@ L<AnyEvent> condvars) the caller until the result is available, so must
 not be called from an event loop callback - see L<EVENT BASED API> for an
 alternative.
 
-The method names are preetty much taken directly from the W3C WebDriver
+The method names are pretty much taken directly from the W3C WebDriver
 specification, e.g. the request documented in the "Get All Cookies"
 section is implemented via the C<get_all_cookies> method.
 
-The order is the same as in the WebDriver draft at the tiome of this
+The order is the same as in the WebDriver draft at the time of this
 writing, and only minimal massaging is done to request parameters and
 results.
 
@@ -212,17 +224,17 @@ results.
 
 =item $wd->new_session ({ key => value... })
 
-Try to connect to a webdriver and initialize session with a "new
-session" command, passing the given key-value pairs as value
+Try to connect to the WebDriver and initialize a new session with a
+"new session" command, passing the given key-value pairs as value
 (e.g. C<capabilities>).
 
 No session-dependent methods must be called before this function returns
-successfully.
+successfully, and only one session can be created per WebDriver object.
 
-On success, C<< $wd->{sid} >> is set to the session id, and C<<
+On success, C<< $wd->{sid} >> is set to the session ID, and C<<
 $wd->{capabilities} >> is set to the returned capabilities.
 
-   my $wd = new AnyEvent::Selenium host => "localhost", port => 4545;
+   my $wd = new AnyEvent::Selenium endpoint => "http://localhost:4545";
 
    $wd->new_session ({
       capabilities => {
@@ -233,17 +245,17 @@ $wd->{capabilities} >> is set to the returned capabilities.
 =cut
 
 sub new_session_ {
-   my ($wdf, $kv, $cb) = @_;
+   my ($self, $kv, $cb) = @_;
 
-   local $wdf->{_ep} = "$wdf->{endpoint}/";
-   $wdf->post_ (session => $kv, sub {
+   local $self->{_ep} = "$self->{endpoint}/";
+   $self->post_ (session => $kv, sub {
       my ($status, $res) = @_;
 
       if ($status eq "200") {
-         $wdf->{sid}          = $res->{sessionId};
-         $wdf->{capabilities} = $res->{capabilities};
+         $self->{sid}          = $res->{sessionId};
+         $self->{capabilities} = $res->{capabilities};
 
-         $wdf->{_ep} = "$wdf->{endpoint}/session/$wdf->{sid}/";
+         $self->{_ep} = "$self->{endpoint}/session/$self->{sid}/";
       }
 
       $cb->($status, $res);
@@ -258,10 +270,10 @@ call.
 =cut
 
 sub delete_session_ {
-   my ($wdf, $cb) = @_;
+   my ($self, $cb) = @_;
 
-   local $wdf->{_ep} = "$wdf->{endpoint}/session/$wdf->{sid}";
-   $wdf->delete_ ("" => $cb);
+   local $self->{_ep} = "$self->{endpoint}/session/$self->{sid}";
+   $self->delete_ ("" => $cb);
 }
 
 =item $timeouts = $wd->get_timeouts
@@ -269,8 +281,7 @@ sub delete_session_ {
 Get the current timeouts, e.g.:
 
    my $timeouts = $wd->get_timeouts;
-
-   # { implicit => 0, pageLoad => 300000, script => 30000 }
+   => { implicit => 0, pageLoad => 300000, script => 30000 }
 
 =item $wd->set_timeouts ($timeouts)
 
@@ -302,7 +313,7 @@ Navigates to the specified URL.
 
 =item $url = $wd->get_current_url
 
-Queries the czurrent page URL as set by C<navigate_to>.
+Queries the current page URL as set by C<navigate_to>.
 
 =cut
 
@@ -396,7 +407,10 @@ sub get_window_handles_ {
 
 =item $handles = $wd->switch_to_frame ($frame)
 
-Switch to the given frame.
+Switch to the given frame identified by C<$frame>, which must be either
+C<undef> to go back to the top-level browsing context, an integer to
+select the nth subframe, or an element object (as e.g. returned by the
+C<element_object> method.
 
 =cut
 
@@ -419,7 +433,7 @@ sub switch_to_parent_frame_ {
 Return the current window rect, e.g.:
 
    $rect = $wd->get_window_rect
-   # { height => 1040, width => 540, x => 0, y => 0 }
+   => { height => 1040, width => 540, x => 0, y => 0 }
 
 =item $wd->set_window_rect ($rect)
 
@@ -441,8 +455,8 @@ sub set_window_rect_ {
 
 =item $wd->fullscreen_window
 
-Changes the window size by eithe3r maximising, minimising or making it
-fullscreen. In my experience, this might timeout if no window manager is
+Changes the window size by either maximising, minimising or making it
+fullscreen. In my experience, this will timeout if no window manager is
 running.
 
 =cut
@@ -470,7 +484,7 @@ sub fullscreen_window_ {
 =item $element_id = $wd->find_element ($location_strategy, $selector)
 
 Finds the first element specified by the given selector and returns its
-web element id (the strong, not the object from the protocol). Raises an
+web element ID (the strong, not the object from the protocol). Raises an
 error when no element was found.
 
    $element = $wd->find_element ("css selector" => "body a");
@@ -478,7 +492,7 @@ error when no element was found.
    $element = $wd->find_element ("partial link text" => "orn");
    $element = $wd->find_element ("tag name" => "input");
    $element = $wd->find_element ("xpath" => '//input[@type="text"]');
-   # "decddca8-5986-4e1d-8c93-efe952505a5f"
+   => e.g. "decddca8-5986-4e1d-8c93-efe952505a5f"
 
 =item $element_ids = $wd->find_elements ($location_strategy, $selector)
 
@@ -558,7 +572,7 @@ Returns the value of the given property.
 
 =item $string = $wd->get_element_css_value ($element_id, $name)
 
-Returns the value of the given css value.
+Returns the value of the given CSS value.
 
 =item $string = $wd->get_element_text ($element_id)
 
@@ -570,7 +584,7 @@ Returns the tag of the given element.
 
 =item $rect = $wd->get_element_rect ($element_id)
 
-Returns the element rect of the given element.
+Returns the element rect(angle) of the given element.
 
 =item $bool = $wd->is_element_enabled
 
@@ -861,7 +875,7 @@ sub send_alert_text_ {
 
 =item $wd->take_screenshot
 
-Create a screenshot, returning it as a PNG image in a data url.
+Create a screenshot, returning it as a PNG image in a C<data:> URL.
 
 =item $wd->take_element_screenshot ($element_id)
 
@@ -885,10 +899,12 @@ sub take_element_screenshot_ {
 
 =cut
 
+=item $object = AnyEvent::WebDriver->element_object ($element_id)
+
 =item $object = $wd->element_object ($element_id)
 
-Encoding element ids in data structures is done by represetning them as an
-object with a special key and the element id as value. This helper method
+Encoding element IDs in data structures is done by representing them as an
+object with a special key and the element ID as value. This helper method
 does this for you.
 
 =cut
@@ -938,7 +954,7 @@ and C<back>, you can make a callback-based ones:
 
 While the blocking methods C<croak> on errors, the callback-based ones all
 pass two values to the callback, C<$status> and C<$res>, where C<$status>
-is the HTTP status code (200 for successful requests, typically 4xx ot
+is the HTTP status code (200 for successful requests, typically 4xx or
 5xx for errors), and C<$res> is the value of the C<value> key in the JSON
 response object.
 
@@ -947,8 +963,8 @@ identical.
 
 =head2 LOW LEVEL API
 
-All the simplfiied API methods are very thin wrappers around WebDriver
-commands of the same name. Theyx are all implemented in terms of the
+All the simplified API methods are very thin wrappers around WebDriver
+commands of the same name. They are all implemented in terms of the
 low-level methods (C<req>, C<get>, C<post> and C<delete>), which exists
 in blocking and callback-based variants (C<req_>, C<get_>, C<post_> and
 C<delete_>).
@@ -1008,7 +1024,7 @@ returned element objects to only return the element ID strings:
 
    my $elems = $wd->post (elements => { using => "css selector", value => "img" });
 
-   # yes, the W3C found an interetsing way around the typelessness of JSON
+   # yes, the W3C found an interesting way around the typelessness of JSON
    $_ = $_->{"element-6066-11e4-a52e-4f735466cecf"}
       for @$elems;
 
